# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-01-26 00:53
from __future__ import unicode_literals

from django.db import migrations, models
import django_discover.fields


class Migration(migrations.Migration):

    dependencies = [
        ('django_discover', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='discoveritem',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='Is active'),
        ),
    ]
