"""
``django-discover`` provides ``DiscoverItem`` model, that allow displaying an introduction
to an existent or a new feature to the user.
"""

__author__ = 'Clément "clemfromspace" Denoix'
__contact__ = 'clement.denoix@magency.fr'
__homepage__ = 'https://gitlab.com/magency/'
__license__ = 'BSD'
__version__ = '1.0.1'
VERSION = tuple(__version__.split('.'))

