"""This module contains the ``DiscoverItem`` admin class"""

from django.contrib import admin

from .models import DiscoverItem


@admin.register(DiscoverItem)
class DiscoverItemAdmin(admin.ModelAdmin):
    """Admin model for the ``DiscoverItem`` model"""

    prepopulated_fields = {'slug': ('title',)}
