"""This module contains the urls for the ``django_discover`` app"""

from django.conf.urls import url

from .views import DiscoverItemsArchiveView

app_name = 'django_discover'  # pylint: disable=invalid-name

urlpatterns = [
    url(
        r'^discover_items/archive/(?P<version>[0-9]+.[0-9]+.[0-9]+)/$',
        DiscoverItemsArchiveView.as_view(),
        name='discover-items-archive'
    ),
]
